﻿/* Settings.cs
 * part of zaaReloaded2
 * 
 * Copyright 2015 Daniel Kraus
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using zaaReloaded2.Formatter;
using zaaReloaded2.Controller.Elements;
using System.IO;
using System.Text.RegularExpressions;
using System.Runtime.Serialization;

namespace zaaReloaded2.Controller
{
    /// <summary>
    /// Holds settings related to controlling laboratory output.
    /// </summary>
    [Serializable]
    public class Settings : ICloneable, ISerializable
    {
        #region Persistence

        /// <summary>
        /// Custom serialization method that writes this settings
        /// to a human-readable text file.
        /// </summary>
        /// <param name="stream"></param>
        public void Persist(Stream stream)
        {
            StreamWriter w = new StreamWriter(stream);
            w.WriteLine(String.Format("[{0}]", Name));

            w.WriteLine(ReferenceStyle.ToString());
            w.Flush();
        }

        public static Settings Unpersist(Stream stream)
        {
            StreamReader r = new StreamReader(stream);
            string line = r.ReadLine();
            Match m = _persistenceHeaderRegex.Match(line);
            if (!m.Success)
                throw new InvalidDataException("Settings header line does not match expected format.");

            Settings settings = new Settings();
            settings.Name = m.Groups["name"].Value;

            line = r.ReadLine();
            settings.ReferenceStyle = (ReferenceStyle)Enum.Parse(typeof(ReferenceStyle), line);

            return settings;
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the name of these settings.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the reference style.
        /// </summary>
        public ReferenceStyle ReferenceStyle { get; set; }

        /// <summary>
        /// Gets the list of controlling elements.
        /// </summary>
        public IList<ElementBase> Elements { get; private set; }

        /// <summary>
        /// Gets the unique ID of this Settings object. The unique
        /// ID is not included in deep-copying (cloning).
        /// </summary>
        public Guid Uid { get; private set; }

        #endregion

        #region Constructors

        public Settings()
            : this(string.Empty, null)
        { }

        public Settings(string name)
            : this(name, null)
        { }

        /// <summary>
        /// Creates a new Settings object with an initial
        /// set of elements.
        /// </summary>
        /// <param name="initialElements">Set of ElementBase
        /// object (or derived ones).</param>
        public Settings(IList<ElementBase> initialElements)
            : this(String.Empty, initialElements)
        { }

        /// <summary>
        /// Creates a new Settings object with an initial
        /// set of elements and a name.
        /// </summary>
        /// <param name="initialElements">Set of ElementBase
        /// object (or derived ones).</param>
        /// <param name="name">Name of these settings.</param>
        public Settings(string name, IList<ElementBase> initialElements)
        {
            Uid = Guid.NewGuid();
            Name = name;
            Elements = initialElements;
            ReferenceStyle = Properties.Settings.Default.ReferenceStyle;
            if (Elements == null)
            {
                Elements = new List<ElementBase>();
            }
        }

        /// <summary>
        /// Creates a new Settings object with an initial
        /// set of elements, a name, and a unique ID.
        /// </summary>
        /// <param name="initialElements">Set of ElementBase
        /// object (or derived ones).</param>
        /// <param name="name">Name of these settings.</param>
        /// <param name="Uid">Unique ID for this Settings object.</param>
        public Settings(string name, IList<ElementBase> initialElements, Guid uid)
            : this(name, initialElements)
        {
            Uid = uid;
        }

        /// <summary>
        /// Deserialization constructor.
        /// </summary>
        protected Settings(SerializationInfo info, StreamingContext context)
        {
            int version = info.GetInt32("Version");
            Uid = (Guid)info.GetValue("Uid", typeof(Guid));
            Name = info.GetString("Name");
            ReferenceStyle = (ReferenceStyle)info.GetValue("ReferenceStyle", typeof(ReferenceStyle));
            int elementsCount = info.GetInt32("ElementsCount");
            Elements = new List<ElementBase>();
            for (int i = 0; i < elementsCount; i++)
            {
                Type type = info.GetValue(SerializationElementName(i, "Type"), typeof(Type)) as Type;
                ElementBase element = info.GetValue(SerializationElementName(i, "Object"), type) as ElementBase;
                Elements.Add(element);
            }
        }   

        #endregion

        #region Public methods

        /// <summary>
        /// Adds an element to the list of Elements.
        /// </summary>
        /// <param name="element">Element to add.</param>
        public void AddElement(ElementBase element)
        {
            Elements.Add(element);
        }

        #endregion

        #region Implementation of ICloneable

        public object Clone()
        {
            Settings clone = new Settings(
                String.Format("Kopie von {0}", Name),
                Elements.Select(e => e.Clone() as ElementBase).ToList()
                );
            clone.ReferenceStyle = ReferenceStyle;
            return clone;
        }

        #endregion

        #region Implementation of ISerializable

        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            info.AddValue("Version", Properties.Settings.Default.SerializationVersion);
            info.AddValue("Uid", Uid);
            info.AddValue("Name", Name);
            info.AddValue("ReferenceStyle", ReferenceStyle);
            info.AddValue("ElementsCount", Elements.Count);
            int i = 0;
            foreach (ElementBase e in Elements)
            {
                info.AddValue(SerializationElementName(i, "Type"), e.GetType());
                info.AddValue(SerializationElementName(i, "Object"), e);
                i++;
            }
        }

        string SerializationElementName(int index, string info)
        {
            return String.Format("Element{0}{1}", index, info);
        }

        #endregion

        #region Fields

        // Defines header lines for the persistence file: "[My name]" and so on.
        static readonly Regex _persistenceHeaderRegex = new Regex(@"^\[(?<name>.+?)]$");

        #endregion
    }
}
